"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CookieCacheManager = void 0;
const helper_1 = require("../helper");
const exception_1 = require("../exception");
class CookieCacheManager {
    constructor(options) {
        var _a;
        this._options = Object.assign({ bucket: (_a = options === null || options === void 0 ? void 0 : options.bucket) !== null && _a !== void 0 ? _a : document }, (options !== null && options !== void 0 ? options : {}));
        this._usedSpace = helper_1.Utils.cookieSpaceUsed(this._options.bucket);
        this._availableSpace = Math.max(helper_1.Defaults.MaxCookieLength - this._usedSpace, 0);
        this.get = this.get.bind(this);
        this.set = this.set.bind(this);
        this.has = this.has.bind(this);
        this.clear = this.clear.bind(this);
        this._resolve = this._resolve.bind(this);
    }
    static getInstance(options) {
        if (!CookieCacheManager.instance)
            CookieCacheManager.instance = new CookieCacheManager(options);
        return CookieCacheManager.instance;
    }
    usedSpace() {
        return this._usedSpace;
    }
    availableSpace() {
        return this._availableSpace;
    }
    calculateSpace(configOrKey, value) {
        const resolve = this._resolve(configOrKey);
        const entryStr = helper_1.Utils.safeStringify({ value, date: new Date() });
        return (resolve.key.length + ((this._options.encryptor ? this._options.encryptor.encrypt(entryStr) : entryStr) + `#_kce_`).length + 2);
    }
    has(configOrKey) {
        return this._resolve(configOrKey).exists;
    }
    get(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return undefined;
        const dirtyValue = helper_1.Utils.getCookie(this._options.bucket, resolve.key);
        const value = dirtyValue.substring(0, dirtyValue.lastIndexOf("#"));
        const parsed = JSON.parse(this._options.encryptor ? this._options.encryptor.decrypt(value) : value);
        return {
            value: parsed.value,
            date: new Date(parsed.date),
        };
    }
    getValue(configOrKey) {
        var _a;
        return ((_a = this.get(configOrKey)) !== null && _a !== void 0 ? _a : {}).value;
    }
    set(configOrKey, value) {
        const spaceAllocated = this.calculateSpace(configOrKey, value);
        if (spaceAllocated > this.availableSpace()) {
            throw new exception_1.NoSufficientCacheSpaceLeftError();
        }
        const resolve = this._resolve(configOrKey);
        const entryStr = helper_1.Utils.safeStringify({ value, date: new Date(), });
        const finalValue = (this._options.encryptor ? this._options.encryptor.encrypt(entryStr) : entryStr) + `#_kce_`;
        if (resolve.exists)
            this.remove(resolve.key);
        helper_1.Utils.addCookie(this._options.bucket, {
            name: resolve.key,
            value: finalValue,
            expires: new Date("Tue, 30 Jul 9024 10:18:20 GMT"),
        });
        this._usedSpace += spaceAllocated;
        this._availableSpace -= spaceAllocated;
    }
    remove(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return;
        const spaceToFree = this.calculateSpace(resolve.key, this.getValue(configOrKey));
        this._usedSpace -= spaceToFree;
        this._availableSpace += spaceToFree;
        helper_1.Utils.removeCookie(this._options.bucket, resolve.key);
    }
    clear() {
        const keys = [];
        helper_1.Utils.getCookieEntries(this._options.bucket, (key, value) => {
            if (`${value}`.endsWith(`#_kce_`))
                keys.push(key);
        });
        for (const key of keys)
            this.remove(key);
    }
    find(cond) {
        const found = [];
        helper_1.Utils.getCookieEntries(this._options.bucket, (key, _) => {
            if (cond(key))
                found.push(key);
        });
        return found;
    }
    _resolve(configOrKey) {
        let key = ((typeof configOrKey === "string") ? configOrKey : helper_1.Utils.buildCacheKey(configOrKey));
        key = (this._options.encryptKey && this._options.encryptor ? this._options.encryptor.encrypt(key) : key);
        const exists = this._options.bucket.cookie.includes(`${key}=`);
        return { key, exists };
    }
}
exports.CookieCacheManager = CookieCacheManager;
