"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StorageCacheManager = void 0;
const helper_1 = require("../helper");
const exception_1 = require("../exception");
class StorageCacheManager {
    constructor(bucket, options) {
        this._options = Object.assign(Object.assign({}, (options !== null && options !== void 0 ? options : {})), { bucket });
        this._usedSpace = helper_1.Utils.storageSpaceUsed(this._options.bucket);
        this._availableSpace = Math.max(helper_1.Defaults.MaxStorageSpace - this._usedSpace, 0);
        this.get = this.get.bind(this);
        this.set = this.set.bind(this);
        this.has = this.has.bind(this);
        this.clear = this.clear.bind(this);
        this._resolve = this._resolve.bind(this);
    }
    usedSpace() {
        return this._usedSpace;
    }
    availableSpace() {
        return this._availableSpace;
    }
    calculateSpace(configOrKey, value) {
        const resolve = this._resolve(configOrKey);
        const entryStr = helper_1.Utils.safeStringify({
            value,
            date: new Date(),
        });
        return (resolve.key.length + ((this._options.encryptor ? this._options.encryptor.encrypt(entryStr) : entryStr) + `#_kce_`).length);
    }
    has(configOrKey) {
        return this._resolve(configOrKey).exists;
    }
    get(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return undefined;
        const dirtyValue = this._options.bucket.getItem(resolve.key);
        const value = dirtyValue.substring(0, dirtyValue.lastIndexOf("#"));
        const parsed = JSON.parse(this._options.encryptor ? this._options.encryptor.decrypt(value) : value);
        return {
            value: parsed.value,
            date: new Date(parsed.date),
        };
    }
    getValue(configOrKey) {
        var _a;
        return ((_a = this.get(configOrKey)) !== null && _a !== void 0 ? _a : {}).value;
    }
    set(configOrKey, value) {
        const resolve = this._resolve(configOrKey);
        const entryStr = helper_1.Utils.safeStringify({
            value,
            date: new Date(),
        });
        const finalValue = (this._options.encryptor ? this._options.encryptor.encrypt(entryStr) : entryStr) + `#_kce_`;
        const spaceAllocated = finalValue.length;
        if (spaceAllocated > this.availableSpace()) {
            throw new exception_1.NoSufficientCacheSpaceLeftError();
        }
        if (resolve.exists)
            this.remove(resolve.key);
        this._options.bucket.setItem(resolve.key, finalValue);
        this._usedSpace += spaceAllocated;
        this._availableSpace -= spaceAllocated;
    }
    remove(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return;
        const spaceToFree = this._options.bucket.getItem(resolve.key).length;
        this._usedSpace -= spaceToFree;
        this._availableSpace += spaceToFree;
        this._options.bucket.removeItem(resolve.key);
    }
    clear() {
        const keys = [];
        helper_1.Utils.getStorageEntries(this._options.bucket, (key, value) => {
            if (`${value}`.endsWith(`#_kce_`))
                keys.push(key);
        });
        for (const key of keys)
            this.remove(key);
    }
    find(cond) {
        const found = [];
        helper_1.Utils.getStorageEntries(this._options.bucket, (key, _) => {
            if (cond(key))
                found.push(key);
        });
        return found;
    }
    _resolve(configOrKey) {
        let key = ((typeof configOrKey === "string") ? configOrKey : helper_1.Utils.buildCacheKey(configOrKey));
        key = (this._options.encryptKey && this._options.encryptor ? this._options.encryptor.encrypt(key) : key);
        const exists = key in this._options.bucket;
        return { key, exists };
    }
}
exports.StorageCacheManager = StorageCacheManager;
