"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventQueue = exports.EventQueueType = void 0;
const helper_1 = require("../helper");
const exception_1 = require("../exception");
exports.EventQueueType = {
    HTTP_REQUEST: "HTTP_REQUEST",
};
class EventQueue {
    constructor() {
        this._Subscriptions = {};
        this._ScheduledExecutions = [];
    }
    static getInstance() {
        if (!EventQueue.instance)
            EventQueue.instance = new EventQueue();
        return EventQueue.instance;
    }
    static executorIsRegistered(type) {
        return (type in EventQueue._RegisteredExecutors);
    }
    static registerExecutor(type, executor) {
        EventQueue._RegisteredExecutors[type] = executor;
    }
    static unregisterExecutor(type) {
        if (!(type in EventQueue._RegisteredExecutors))
            return;
        delete EventQueue._RegisteredExecutors[type];
    }
    static registerPreExecutor(type, preExecutor) {
        EventQueue._RegisteredPreExecutors[type] = preExecutor;
    }
    static unregisterPreExecutor(type) {
        if (!(type in EventQueue._RegisteredPreExecutors))
            return;
        delete EventQueue._RegisteredPreExecutors[type];
    }
    queueEvent(cache, event, key) {
        const id = this._generateId(key);
        if (event.retryIn && !event.date) {
            event.date = new Date();
        }
        cache.set(this._buildId(id), event);
        return id;
    }
    queue(cache, type, ...params) {
        return this.queueEvent(cache, { type, params });
    }
    async executeOnly(cache, id, type) {
        const event = cache.getValue(this._buildId(id));
        if (event === undefined) {
            throw new exception_1.NoEventFoundWithIdError(id);
        }
        if (type && event.type !== type)
            return;
        if (event.date && event.retryIn) {
            if (this._ScheduledExecutions.includes(id)) {
                return "___KF_ALREADY_SCHEDULED_FOR_TIMED_EXECUTION___";
            }
            if (typeof event.date === "string") {
                event.date = new Date(event.date);
            }
            event.date = helper_1.Utils.addMsToDate(event.date, event.retryIn);
            const executionTimeMs = helper_1.Utils.dateDiffInMs(new Date(), event.date);
            if (executionTimeMs > 0) {
                this._ScheduledExecutions.push(id);
                setTimeout((() => {
                    this.executeOnly(cache, id, type);
                }).bind(this), executionTimeMs);
                return "___KF_SCHEDULED_FOR_TIMED_EXECUTION___";
            }
        }
        cache.remove(this._buildId(id));
        const params = (event.type in EventQueue._RegisteredPreExecutors ? EventQueue._RegisteredPreExecutors[event.type](cache, ...event.params) : event.params);
        const result = await (EventQueue._RegisteredExecutors[event.type](...params));
        if (event.subscriptionKey)
            this.report(event.subscriptionKey, result);
        return result;
    }
    async execute(cache, type) {
        const results = [];
        const keys = cache.find((key) => key.startsWith(`KQ__`));
        for (const key of keys) {
            const result = await this.executeOnly(cache, key.replace(`KQ__`, ""), type);
            if (result === "___KF_ALREADY_SCHEDULED_FOR_TIMED_EXECUTION___")
                continue;
            results.push(result);
        }
        return results;
    }
    subscribe(key, subscription) {
        if (!(key in this._Subscriptions))
            this._Subscriptions[key] = [];
        if (this._Subscriptions[key].includes(subscription))
            return;
        this._Subscriptions[key].push(subscription);
    }
    unSubscribe(key, subscription) {
        if (!(key in this._Subscriptions))
            return;
        const index = this._Subscriptions[key].indexOf(subscription);
        if (index > -1)
            this._Subscriptions[key].splice(index, 1);
    }
    report(key, ...data) {
        if (!(key in this._Subscriptions))
            return;
        this._Subscriptions[key].forEach(subscription => subscription(...data));
    }
    subscriptions(key) {
        if (key)
            return this._Subscriptions[key];
        return Object.values(this._Subscriptions);
    }
    clearSubscriptions(key) {
        if (key) {
            if (key in this._Subscriptions) {
                delete this._Subscriptions[key];
            }
            return;
        }
        this._Subscriptions = {};
    }
    _generateId(key) {
        return (key !== null && key !== void 0 ? key : helper_1.Utils.randomString(20, "0123456789"));
    }
    _buildId(id) {
        return `KQ__${id}`;
    }
}
exports.EventQueue = EventQueue;
EventQueue._RegisteredExecutors = {};
EventQueue._RegisteredPreExecutors = {};
