"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Http = void 0;
const Interceptor_1 = require("./Interceptor");
const EventQueue_1 = require("./EventQueue");
const helper_1 = require("../helper");
const types_1 = require("../types");
const exception_1 = require("../exception");
const cachemanager_1 = require("../cachemanager");
class Http {
    constructor(baseConfig) {
        this._queueRebuilds = { cache: "__kyofuuc_rebuild__" };
        this._nonSerializable = ["interceptor", "connector", "parsed", "httpAgent", "httpsAgent", "validateStatus", "querySerializer", "transport"];
        this._baseConfig = baseConfig !== null && baseConfig !== void 0 ? baseConfig : {};
        this.queueRequest = this.queueRequest.bind(this);
        this.updateConfig = this.updateConfig.bind(this);
        this.retryRequests = this.retryRequests.bind(this);
    }
    static getInstance(baseConfig) {
        if (!Http.instance)
            Http.instance = new Http(baseConfig);
        return Http.instance;
    }
    getConfig() {
        return this._baseConfig;
    }
    updateConfig(newConfig, merge) {
        this._baseConfig = merge ? Object.assign(Object.assign({}, this._baseConfig), newConfig) : newConfig;
    }
    getUrl(config) {
        var _a;
        if (!config.url) {
            throw new exception_1.InvalidParameterError("Invalid or missing url in config");
        }
        const mergedConfig = helper_1.Utils.mergeObjects(this._baseConfig, config, ["url", "baseUrl", "query", "querySerializer"]);
        return (_a = helper_1.Utils.buildUrlWithQueryFromConfig({
            query: mergedConfig.query,
            url: helper_1.Utils.buildFullUrl(mergedConfig),
            querySerializer: mergedConfig.querySerializer,
        })) !== null && _a !== void 0 ? _a : config.url;
    }
    request(urlOrConfig, config) {
        const interceptors = [];
        if (typeof urlOrConfig === 'string') {
            config = Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { url: urlOrConfig });
        }
        else {
            config = Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), (urlOrConfig !== null && urlOrConfig !== void 0 ? urlOrConfig : {}));
        }
        if (config === null || config === void 0 ? void 0 : config.interceptor)
            interceptors.push(config.interceptor);
        if (this._baseConfig.interceptor)
            interceptors.push(this._baseConfig.interceptor);
        const mergedConfig = helper_1.Defaults.httpConfig(helper_1.Utils.mergeObjects(this._baseConfig, config !== null && config !== void 0 ? config : {}));
        const url = this.getUrl(mergedConfig);
        mergedConfig.parsed = helper_1.Utils.parseUrl(url);
        mergedConfig.url = helper_1.Utils.stripAuthFromUrl(url);
        mergedConfig.interceptor = new Interceptor_1.Interceptor(interceptors);
        if (mergedConfig.parsed.username || mergedConfig.parsed.password) {
            mergedConfig.auth = { username: mergedConfig.parsed.username, password: mergedConfig.parsed.password };
        }
        this._registerCacheInterceptors(mergedConfig);
        return mergedConfig.connector(mergedConfig, this.queueRequest);
    }
    get(url, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.GET }));
    }
    head(url, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.HEAD }));
    }
    delete(url, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.DELETE }));
    }
    options(url, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.OPTIONS }));
    }
    put(url, data, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.PUT, data }));
    }
    post(url, data, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.POST, data }));
    }
    patch(url, data, config) {
        return this.request(url, Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { method: types_1.Method.PATCH, data }));
    }
    queueRequest(config) {
        if (!config.cacheManager) {
            throw new exception_1.MissingCacheError();
        }
        EventQueue_1.EventQueue.getInstance().queueEvent(config.cacheManager, {
            type: EventQueue_1.EventQueueType.HTTP_REQUEST,
            subscriptionKey: config.subscriptionKey,
            params: [helper_1.Utils.cherryPick(config, this._nonSerializable, true, this._queueRebuilds)],
        }, config.key);
    }
    async retryRequests(cacheManager) {
        if (cacheManager) {
            EventQueue_1.EventQueue.getInstance().execute(cacheManager, EventQueue_1.EventQueueType.HTTP_REQUEST);
            return;
        }
        EventQueue_1.EventQueue.getInstance().execute(cachemanager_1.MapCacheManager.getInstance(), EventQueue_1.EventQueueType.HTTP_REQUEST);
        EventQueue_1.EventQueue.getInstance().execute(cachemanager_1.LocalStorageCacheManager.getInstance(), EventQueue_1.EventQueueType.HTTP_REQUEST);
        EventQueue_1.EventQueue.getInstance().execute(cachemanager_1.SessionStorageCacheManager.getInstance(), EventQueue_1.EventQueueType.HTTP_REQUEST);
    }
    _registerCacheInterceptors(config) {
        var _a, _b;
        if (!config.cacheManager)
            return;
        if (!config.refreshCache) {
            (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.registerPreRequest((config) => {
                var _a, _b;
                if (config === null || config === void 0 ? void 0 : config.cache) {
                    if ((typeof (config === null || config === void 0 ? void 0 : config.cache) === "function" && !(config === null || config === void 0 ? void 0 : config.cache(config, "REQUEST"))) || (typeof (config === null || config === void 0 ? void 0 : config.cache) === "boolean" && !(config === null || config === void 0 ? void 0 : config.cache)))
                        return;
                }
                const cached = (_a = config === null || config === void 0 ? void 0 : config.cacheManager) === null || _a === void 0 ? void 0 : _a.get(config);
                if (!cached || ((config === null || config === void 0 ? void 0 : config.cacheLifetime) && (cached === null || cached === void 0 ? void 0 : cached.date) && ((new Date()).getTime() - cached.date.getTime()) >= (config === null || config === void 0 ? void 0 : config.cacheLifetime))) {
                    if (cached && !(config === null || config === void 0 ? void 0 : config.persistCache))
                        (_b = config === null || config === void 0 ? void 0 : config.cacheManager) === null || _b === void 0 ? void 0 : _b.remove(config);
                    return;
                }
                return Object.assign(Object.assign({}, cached === null || cached === void 0 ? void 0 : cached.value), { __cached__: true, __cached_expires_at__: cached === null || cached === void 0 ? void 0 : cached.date });
            });
        }
        (_b = config.interceptor) === null || _b === void 0 ? void 0 : _b.registerPostResponse((config, _, response) => {
            var _a;
            if ((typeof (config === null || config === void 0 ? void 0 : config.cache) === "function" && !(config === null || config === void 0 ? void 0 : config.cache(config, "RESPONSE", response))) || (typeof (config === null || config === void 0 ? void 0 : config.cache) === "boolean" && !(config === null || config === void 0 ? void 0 : config.cache)))
                return;
            if ((response === null || response === void 0 ? void 0 : response.__cached__) || (response === null || response === void 0 ? void 0 : response.status) === 150)
                return;
            (_a = config === null || config === void 0 ? void 0 : config.cacheManager) === null || _a === void 0 ? void 0 : _a.set(config, response);
        });
    }
}
exports.Http = Http;
if (!EventQueue_1.EventQueue.executorIsRegistered(EventQueue_1.EventQueueType.HTTP_REQUEST)) {
    EventQueue_1.EventQueue.registerExecutor(EventQueue_1.EventQueueType.HTTP_REQUEST, Http.getInstance().request.bind(Http.getInstance()));
    EventQueue_1.EventQueue.registerPreExecutor(EventQueue_1.EventQueueType.HTTP_REQUEST, (cacheManager, config) => {
        config.cacheManager = cacheManager;
        return [config];
    });
}
