"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Interceptor = exports.HandlerType = void 0;
const helper_1 = require("../helper");
var HandlerType;
(function (HandlerType) {
    HandlerType["CUSTOM"] = "CUSTOM";
    HandlerType["WS_OPEN"] = "WS_OPEN";
    HandlerType["WS_CLOSE"] = "WS_CLOSE";
    HandlerType["WS_ERROR"] = "WS_ERROR";
    HandlerType["WS_MESSAGE"] = "WS_MESSAGE";
    HandlerType["WS_STATE_CHANGE"] = "WS_STATE_CHANGE";
    HandlerType["HTTP_PRE_REQUEST"] = "HTTP_PRE_REQUEST";
    HandlerType["HTTP_POST_REQUEST"] = "HTTP_POST_REQUEST";
    HandlerType["HTTP_PRE_RESPONSE"] = "HTTP_PRE_RESPONSE";
    HandlerType["HTTP_POST_RESPONSE"] = "HTTP_POST_RESPONSE";
    HandlerType["HTTP_REQUEST_MAXIMUM_REDIRECTS_REACHED"] = "HTTP_REQUEST_MAXIMUM_REDIRECTS_REACHED";
})(HandlerType || (exports.HandlerType = HandlerType = {}));
class Interceptor {
    constructor(interceptors) {
        this._count = 0;
        this._handlers = {};
        this._reverse_handler_type_map = {};
        if (interceptors === null || interceptors === void 0 ? void 0 : interceptors.length) {
            this.registers(interceptors);
        }
    }
    count() {
        return this._count;
    }
    register(handler) {
        const type = `${handler.type}`;
        const key = `${handler.type}_${helper_1.Utils.randomString()}`;
        if (!(type in this._handlers)) {
            this._handlers[type] = {};
        }
        this._reverse_handler_type_map[key] = type;
        this._handlers[type][key] = handler;
        this._count++;
        return key;
    }
    unregister(key) {
        if (!(key in this._reverse_handler_type_map))
            return;
        const type = this._reverse_handler_type_map[key];
        delete this._reverse_handler_type_map[key];
        delete this._handlers[type][key];
        this._count--;
    }
    purge() {
        this._count = 0;
        this._handlers = {};
        this._reverse_handler_type_map = {};
    }
    registers(interceptors) {
        const keys = [];
        helper_1.Utils.forEach(interceptors, (_, interceptor) => {
            helper_1.Utils.forEach(interceptor._handlers, (_, valueMap) => {
                helper_1.Utils.forEach(valueMap, (__, value) => {
                    keys.push(this.register(value));
                });
            });
        });
        return keys;
    }
    handler(key) {
        if (!(key in this._reverse_handler_type_map))
            return;
        const type = this._reverse_handler_type_map[key];
        return this._handlers[type][key];
    }
    handlers(type) {
        var _a;
        return Object.values((_a = this._handlers[`${type}`]) !== null && _a !== void 0 ? _a : {});
    }
    invoke(type, config, response) {
        var _a;
        const results = [];
        const handlers = Object.values((_a = this._handlers[`${type}`]) !== null && _a !== void 0 ? _a : {});
        for (const handler of handlers) {
            if (handler.when && !handler.when(config))
                continue;
            const result = handler.cb(config, handler.options, response);
            if (result === null || result === undefined)
                continue;
            results.push(result);
        }
        return results;
    }
    registerPreRequest(cb, options) {
        return this.register({ cb, options, type: HandlerType.HTTP_PRE_REQUEST, });
    }
    registerPostRequest(cb, options) {
        return this.register({ cb, options, type: HandlerType.HTTP_POST_REQUEST, });
    }
    registerPreResponse(cb, options) {
        return this.register({ cb, options, type: HandlerType.HTTP_PRE_RESPONSE, });
    }
    registerPostResponse(cb, options) {
        return this.register({ cb, options, type: HandlerType.HTTP_POST_RESPONSE, });
    }
    registerOnWsOpen(cb, options) {
        return this.register({ cb, options, type: HandlerType.WS_OPEN, });
    }
    registerOnWsClose(cb, options) {
        return this.register({ cb, options, type: HandlerType.WS_CLOSE, });
    }
    registerOnWsError(cb, options) {
        return this.register({ cb, options, type: HandlerType.WS_ERROR, });
    }
    registerOnWsMessage(cb, options) {
        return this.register({ cb, options, type: HandlerType.WS_MESSAGE, });
    }
    registerOnWsStateChange(cb, options) {
        return this.register({ cb, options, type: HandlerType.WS_STATE_CHANGE, });
    }
}
exports.Interceptor = Interceptor;
