"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ws = exports.WsState = void 0;
const Interceptor_1 = require("./Interceptor");
const helper_1 = require("../helper");
const exception_1 = require("../exception");
const types_1 = require("../types");
var WsState;
(function (WsState) {
    WsState["IDLE"] = "IDLE";
    WsState["READY"] = "READY";
    WsState["CONNECTED"] = "READY";
    WsState["CONNECTING"] = "CONNECTING";
    WsState["DISCONNECTED"] = "DISCONNECTED";
    WsState["RECONNECTING"] = "RECONNECTING";
    WsState["DISCONNECTING"] = "DISCONNECTING";
    WsState["PROCESSING_OUTGOING_MESSAGE"] = "PROCESSING_OUTGOING_MESSAGE";
    WsState["PROCESSING_INCOMING_MESSAGE"] = "PROCESSING_INCOMING_MESSAGE";
})(WsState || (exports.WsState = WsState = {}));
class Ws {
    constructor(baseConfig) {
        this._state = WsState.IDLE;
        this._reconnectionCount = 0;
        this._lastReconnectionCount = 0;
        this._nextReconnectionDelay = 0;
        this._baseConfig = baseConfig !== null && baseConfig !== void 0 ? baseConfig : {};
        this._registerInterceptors = this._registerInterceptors.bind(this);
    }
    static getInstance(baseConfig) {
        if (!Ws.instance)
            Ws.instance = new Ws(baseConfig);
        return Ws.instance;
    }
    static ws(config) {
        return (new Ws()).connect(config);
    }
    getConfig() {
        return this._baseConfig;
    }
    updateConfig(newConfig, merge) {
        this._baseConfig = merge ? Object.assign(Object.assign({}, this._baseConfig), newConfig) : newConfig;
    }
    getUrl(config) {
        var _a;
        if (!config) {
            config = this._config;
        }
        if (!(config === null || config === void 0 ? void 0 : config.url)) {
            throw new exception_1.InvalidParameterError("Invalid or missing url in config");
        }
        const mergedConfig = helper_1.Utils.mergeObjects(this._baseConfig, config, ["url", "baseUrl", "query", "querySerializer"]);
        return (_a = helper_1.Utils.buildUrlWithQueryFromConfig({
            query: mergedConfig.query,
            url: helper_1.Utils.buildFullUrl(mergedConfig),
            querySerializer: mergedConfig.querySerializer,
        })) !== null && _a !== void 0 ? _a : config.url;
    }
    connect(urlOrConfig, config) {
        var _a;
        const interceptors = [];
        if (typeof urlOrConfig === 'string') {
            config = Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), { url: urlOrConfig });
        }
        else {
            config = Object.assign(Object.assign({}, (config !== null && config !== void 0 ? config : {})), (urlOrConfig !== null && urlOrConfig !== void 0 ? urlOrConfig : {}));
        }
        if (this._cachedInterceptor) {
            interceptors.push(this._cachedInterceptor);
        }
        else {
            this._state = WsState.CONNECTING;
            if (config === null || config === void 0 ? void 0 : config.interceptor)
                interceptors.push(config.interceptor);
            if (this._baseConfig.interceptor)
                interceptors.push(this._baseConfig.interceptor);
        }
        this._config = helper_1.Defaults.wsConfig(helper_1.Utils.mergeObjects(this._baseConfig, config !== null && config !== void 0 ? config : {}));
        this._config.url = this.getUrl(this._config);
        this._config.interceptor = new Interceptor_1.Interceptor(interceptors);
        if (this._cachedInterceptor) {
            this._cachedInterceptor.purge();
            this._cachedInterceptor = undefined;
        }
        else {
            this._registerInterceptors();
        }
        if (this._state === WsState.CONNECTING)
            (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
        this._connection = this._config.connector(this._config);
        this._connectorName = this._connection.name;
        return this;
    }
    reconnect(connector) {
        var _a;
        if (this._isConnected()) {
            if (this._config) {
                this._cachedInterceptor = new Interceptor_1.Interceptor([this._config.interceptor]);
            }
            this.close();
        }
        this._state = WsState.RECONNECTING;
        (_a = this._cachedInterceptor) === null || _a === void 0 ? void 0 : _a.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
        if (connector)
            this._config.connector = connector;
        return this.connect(this._config);
    }
    sendMessage(message) {
        var _a;
        if (!this._isConnected()) {
            throw new exception_1.ConnectionClosedError();
        }
        let dataTransformationFailed = false;
        this._state = WsState.PROCESSING_OUTGOING_MESSAGE;
        let data = (0, types_1.transformRequestData)(this._config, message, false, {}, (error) => {
            var _a, _b;
            (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(Interceptor_1.HandlerType.WS_ERROR, this._config, error);
            this._state = WsState.READY;
            dataTransformationFailed = true;
        });
        if (dataTransformationFailed)
            return;
        (_a = this._connection) === null || _a === void 0 ? void 0 : _a.send(data);
        this._state = WsState.READY;
    }
    close() {
        var _a, _b, _c, _d, _e;
        if (!this._isConnected())
            return;
        this._state = WsState.DISCONNECTING;
        (_a = this._cachedInterceptor) === null || _a === void 0 ? void 0 : _a.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
        if (this._config) {
            (_c = (_b = this._config) === null || _b === void 0 ? void 0 : _b.interceptor) === null || _c === void 0 ? void 0 : _c.purge();
            this._config.interceptor = undefined;
        }
        if (this._messageInterceptor) {
            (_d = this._messageInterceptor) === null || _d === void 0 ? void 0 : _d.purge();
            this._messageInterceptor = undefined;
        }
        (_e = this._connection) === null || _e === void 0 ? void 0 : _e.close();
        this._connection = undefined;
    }
    getConnection() {
        return this._connection;
    }
    getBinaryType() {
        if (!this._isConnected())
            return;
        if (this._connectorName === "wsConnector") {
            return this._connection.binaryType;
        }
    }
    getBufferedAmount() {
        var _a;
        if (!this._isConnected())
            return -1;
        if (this._connectorName === "wsConnector") {
            return (_a = this._connection.bufferedAmount) !== null && _a !== void 0 ? _a : -1;
        }
        return -1;
    }
    getState() {
        return this._state;
    }
    getLastReconnectionCount() {
        return this._lastReconnectionCount;
    }
    setBinaryType(binaryType) {
        if (!this._isConnected())
            return;
        if (this._connectorName === "wsConnector") {
            this._connection.binaryType = binaryType;
        }
    }
    onOpen(cb, options) {
        var _a, _b;
        if (!this._isConnected())
            throw new exception_1.ConnectionClosedError();
        (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.registerOnWsOpen(((_, options, event) => cb(this, event, options)).bind(this), options);
    }
    onClose(cb, options, always) {
        var _a, _b;
        if (!this._isConnected())
            throw new exception_1.ConnectionClosedError();
        (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.registerOnWsClose(((_, options, event) => {
            if (!always && this._state !== WsState.DISCONNECTED)
                return;
            cb(this, event, options);
        }).bind(this), options);
    }
    onError(cb, options) {
        var _a, _b;
        if (!this._isConnected())
            throw new exception_1.ConnectionClosedError();
        (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.registerOnWsError(((_, options, event) => cb(this, event, options)).bind(this), options);
    }
    onStateChange(cb, options) {
        var _a, _b;
        if (!this._isConnected())
            throw new exception_1.ConnectionClosedError();
        (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.registerOnWsStateChange(((_, options, event) => cb(this, event, options)).bind(this), options);
    }
    onMessage(cb, options) {
        var _a;
        if (!this._isConnected())
            throw new exception_1.ConnectionClosedError();
        if (!this._messageInterceptor) {
            this._messageInterceptor = new Interceptor_1.Interceptor();
        }
        (_a = this._messageInterceptor) === null || _a === void 0 ? void 0 : _a.registerOnWsMessage(((_, options, event) => {
            cb(this, event.event, event.message, options);
        }).bind(this), options);
    }
    _isConnected() {
        return !(this._state === WsState.IDLE || this._state === WsState.DISCONNECTING || this._state === WsState.DISCONNECTED);
    }
    _registerInterceptors() {
        var _a, _b;
        if (!this._config.protocol || this._config.protocol === "text") {
            this._config.requestType = types_1.RequestType.TEXT;
            this._config.responseType = types_1.RequestType.TEXT;
        }
        else if (this._config.protocol === "json") {
            this._config.requestType = types_1.RequestType.JSON;
            this._config.responseType = types_1.RequestType.JSON;
        }
        this.onOpen(((_, __, ___) => {
            var _a, _b;
            this._reconnectionCount = 0;
            this._state = WsState.READY;
            this._nextReconnectionDelay = 0;
            (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
        }).bind(this));
        this.onClose(((_, __, ___) => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            if (this._state !== WsState.DISCONNECTING && ((_a = this._config) === null || _a === void 0 ? void 0 : _a.reconnect)) {
                if (this._lastReconnectionCount < ((_c = (_b = this._config) === null || _b === void 0 ? void 0 : _b.maxReconnect) !== null && _c !== void 0 ? _c : 0)) {
                    this._reconnectionCount++;
                    this._lastReconnectionCount++;
                    this._nextReconnectionDelay = (_f = (((_d = this._config) === null || _d === void 0 ? void 0 : _d.reconnectIntervalByPower)
                        ? Math.pow(((_e = this._config.reconnectInterval) !== null && _e !== void 0 ? _e : 0), this._reconnectionCount)
                        : this._config.reconnectInterval)) !== null && _f !== void 0 ? _f : 0;
                    setTimeout(this.reconnect.bind(this), this._nextReconnectionDelay * 1000);
                    return;
                }
            }
            this._state = WsState.DISCONNECTED;
            (_h = (_g = this._config) === null || _g === void 0 ? void 0 : _g.interceptor) === null || _h === void 0 ? void 0 : _h.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
        }).bind(this), undefined, true);
        (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.registerOnWsMessage(((_, __, message) => {
            var _a, _b;
            this._state = WsState.PROCESSING_OUTGOING_MESSAGE;
            (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
            (0, types_1.transformResponseData)(this._config, this._config, message.data, (result) => {
                var _a, _b, _c;
                this._state = WsState.READY;
                (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
                (_c = this._messageInterceptor) === null || _c === void 0 ? void 0 : _c.invoke(Interceptor_1.HandlerType.WS_MESSAGE, this._config, { event: message, message: result.data });
            }, (error) => {
                var _a, _b, _c, _d;
                this._state = WsState.READY;
                (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(Interceptor_1.HandlerType.WS_ERROR, this._config, error);
                (_d = (_c = this._config) === null || _c === void 0 ? void 0 : _c.interceptor) === null || _d === void 0 ? void 0 : _d.invoke(Interceptor_1.HandlerType.WS_STATE_CHANGE, this._config, this._state);
            });
        }).bind(this));
    }
}
exports.Ws = Ws;
