"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompressionProcessor = exports.CompressionType = void 0;
exports.gzipCompressionTransformer = gzipCompressionTransformer;
exports.inflateCompressionTransformer = inflateCompressionTransformer;
exports.deflateCompressionTransformer = deflateCompressionTransformer;
exports.gunzipCompressionTransformer = gunzipCompressionTransformer;
exports.brotliCompressionTransformer = brotliCompressionTransformer;
exports.brotliDecompressionTransformer = brotliDecompressionTransformer;
const zlib_1 = undefined;
const exception_1 = require("../exception");
exports.CompressionType = {
    GZIP: "gzip",
    UNZIP: "unzip",
    BROTLI: "brotli",
    GUNZIP: "gunzip",
    INFLATE: "inflate",
    DEFLATE: "deflate",
    COMPRESS: "compress",
};
function gzipCompressionTransformer() {
    return zlib_1.default.createGzip();
}
function inflateCompressionTransformer() {
    return zlib_1.default.createInflate();
}
function deflateCompressionTransformer() {
    return zlib_1.default.createUnzip();
}
function gunzipCompressionTransformer() {
    return zlib_1.default.createGunzip();
}
function brotliCompressionTransformer() {
    return zlib_1.default.createBrotliCompress();
}
function brotliDecompressionTransformer() {
    return zlib_1.default.createBrotliDecompress();
}
exports.CompressionProcessor = {
    _RegisteredCompressTransformers: {},
    _RegisteredDecompressTransformers: {},
    register(type, flow, transformer) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (flow === "compress") {
            exports.CompressionProcessor._RegisteredCompressTransformers[type] = transformer;
            return;
        }
        exports.CompressionProcessor._RegisteredDecompressTransformers[type.toUpperCase()] = transformer;
    },
    unregister(type, flow) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (flow === "compress") {
            if (!(type in exports.CompressionProcessor._RegisteredCompressTransformers))
                return;
            delete exports.CompressionProcessor._RegisteredCompressTransformers[type];
            return;
        }
        if (!(type in exports.CompressionProcessor._RegisteredDecompressTransformers))
            return;
        delete exports.CompressionProcessor._RegisteredDecompressTransformers[type];
    },
    transform(type, flow) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (flow === "compress") {
            if (!(type in exports.CompressionProcessor._RegisteredCompressTransformers)) {
                throw new exception_1.UnregisteredCompressionTypeError(type, flow);
            }
            const transformer = exports.CompressionProcessor._RegisteredCompressTransformers[type];
            if (!transformer) {
                throw new exception_1.UnexpectedError(`The compression compress type '${type}' found but with invalid transformer`);
            }
            return transformer();
        }
        if (!(type in exports.CompressionProcessor._RegisteredDecompressTransformers)) {
            throw new exception_1.UnregisteredCompressionTypeError(type, flow);
        }
        const transformer = exports.CompressionProcessor._RegisteredDecompressTransformers[type];
        if (!transformer) {
            throw new exception_1.UnexpectedError(`The compression decompress type '${type}' found but with invalid transformer`);
        }
        return transformer();
    },
};
let _defaultCompressionTransformerRegistered = false;
if (!_defaultCompressionTransformerRegistered) {
    _defaultCompressionTransformerRegistered = true;
    exports.CompressionProcessor.register(exports.CompressionType.GZIP, "compress", gzipCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.GUNZIP, "compress", gunzipCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.BROTLI, "compress", brotliCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.INFLATE, "compress", inflateCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.COMPRESS, "compress", inflateCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.UNZIP, "decompress", deflateCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.DEFLATE, "decompress", deflateCompressionTransformer);
    exports.CompressionProcessor.register(exports.CompressionType.BROTLI, "decompress", brotliDecompressionTransformer);
}
