"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestProcessor = exports.RequestType = void 0;
exports.textRequestTransformer = textRequestTransformer;
exports.jsonRequestTransformer = jsonRequestTransformer;
exports.bufferRequestTransformer = bufferRequestTransformer;
exports.arrayBufferRequestTransformer = arrayBufferRequestTransformer;
exports.urlSearchParamsRequestTransformer = urlSearchParamsRequestTransformer;
exports.formDataRequestTransformer = formDataRequestTransformer;
exports.streamRequestTransformer = streamRequestTransformer;
exports.transformRequestData = transformRequestData;
const buffer_1 = require("buffer");
const helper_1 = require("../helper");
const exception_1 = require("../exception");
exports.RequestType = {
    TEXT: "TEXT",
    JSON: "JSON",
    STREAM: "STREAM",
    BUFFER: "BUFFER",
    FORM_DATA: "FORM_DATA",
    ARRAY_BUFFER: "ARRAY_BUFFER",
    URL_SEARCH_PARAMS: "URL_SEARCH_PARAMS",
};
function textRequestTransformer(data) {
    return {
        contentType: "text/html",
        buffer: buffer_1.Buffer.from(data, 'utf-8'),
    };
}
function jsonRequestTransformer(data) {
    return {
        contentType: "application/json",
        buffer: buffer_1.Buffer.from(helper_1.Utils.safeStringify(data), 'utf-8'),
    };
}
function bufferRequestTransformer(data) {
    return {
        buffer: data,
        contentType: "application/octet-stream",
    };
}
function arrayBufferRequestTransformer(data) {
    return bufferRequestTransformer(buffer_1.Buffer.from(new Uint8Array(data)));
}
function urlSearchParamsRequestTransformer(data) {
    return {
        buffer: buffer_1.Buffer.from(data.toString(), 'utf-8'),
        contentType: "application/x-www-form-urlencoded",
    };
}
function formDataRequestTransformer(data) {
    throw new Error("Not implemented");
    return {
        buffer: data,
        contentType: "application/octet-stream",
    };
}
function streamRequestTransformer(data) {
    return bufferRequestTransformer(data);
}
exports.RequestProcessor = {
    _RegisteredTransformers: {},
    register(type, transformer) {
        exports.RequestProcessor._RegisteredTransformers[type.toUpperCase()] = transformer;
    },
    unregister(type) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (!(type in exports.RequestProcessor._RegisteredTransformers))
            return;
        delete exports.RequestProcessor._RegisteredTransformers[type];
    },
    transform(type, data) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (!(type in exports.RequestProcessor._RegisteredTransformers)) {
            throw new exception_1.UnregisteredRequestTypeError(type);
        }
        const transformer = exports.RequestProcessor._RegisteredTransformers[type];
        if (!transformer) {
            throw new exception_1.UnexpectedError(`The request type '${type}' found but with invalid transformer`);
        }
        return transformer(data);
    },
};
function transformRequestData(config, data, contentTypeNotSet, headers, reject) {
    try {
        const requestDataTransformed = exports.RequestProcessor.transform(config.requestType, data);
        if (contentTypeNotSet) {
            headers['Content-Type'] = requestDataTransformed.contentType;
        }
        return requestDataTransformed.buffer;
    }
    catch (err) {
        reject(helper_1.Utils.kyofuucError(err.message, config, helper_1.ErrorCode.REQUEST_DATA_TRANSFORMATION_FAILED));
    }
}
let _defaultRequestTransformerRegistered = false;
if (!_defaultRequestTransformerRegistered) {
    _defaultRequestTransformerRegistered = true;
    exports.RequestProcessor.register(exports.RequestType.TEXT, textRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.JSON, jsonRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.STREAM, streamRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.BUFFER, bufferRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.FORM_DATA, formDataRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.ARRAY_BUFFER, arrayBufferRequestTransformer);
    exports.RequestProcessor.register(exports.RequestType.URL_SEARCH_PARAMS, urlSearchParamsRequestTransformer);
}
