"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResponseProcessor = exports.ResponseType = void 0;
exports.textResponseTransformer = textResponseTransformer;
exports.jsonResponseTransformer = jsonResponseTransformer;
exports.streamResponseTransformer = streamResponseTransformer;
exports.bufferResponseTransformer = bufferResponseTransformer;
exports.arrayBufferResponseTransformer = arrayBufferResponseTransformer;
exports.noContentResponseTransformer = noContentResponseTransformer;
exports.transformResponseData = transformResponseData;
const helper_1 = require("../helper");
const exception_1 = require("../exception");
exports.ResponseType = {
    TEXT: "TEXT",
    JSON: "JSON",
    STREAM: "STREAM",
    BUFFER: "BUFFER",
    NOCONTENT: "NOCONTENT",
    ARRAY_BUFFER: "ARRAY_BUFFER",
};
function textResponseTransformer(data) {
    return { data };
}
function jsonResponseTransformer(data) {
    return { data: JSON.parse(data), };
}
function streamResponseTransformer(data) {
    return { data };
}
function bufferResponseTransformer(data) {
    return { data: Buffer.from(data), };
}
function arrayBufferResponseTransformer(data) {
    return { data };
}
function noContentResponseTransformer(_) {
    return { data: "" };
}
exports.ResponseProcessor = {
    _RegisteredTransformers: {},
    register(type, transformer) {
        exports.ResponseProcessor._RegisteredTransformers[type.toUpperCase()] = transformer;
    },
    unregister(type) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (!(type in exports.ResponseProcessor._RegisteredTransformers))
            return;
        delete exports.ResponseProcessor._RegisteredTransformers[type];
    },
    transform(type, data) {
        type = type === null || type === void 0 ? void 0 : type.toUpperCase();
        if (!(type in exports.ResponseProcessor._RegisteredTransformers)) {
            throw new exception_1.UnregisteredResponseTypeError(type);
        }
        const transformer = exports.ResponseProcessor._RegisteredTransformers[type];
        if (!transformer) {
            throw new exception_1.UnexpectedError(`The response type '${type}' found but with invalid transformer`);
        }
        return transformer(data);
    },
};
function transformResponseData(config, response, res, onSuccess, onError) {
    try {
        const responseDataTransformed = exports.ResponseProcessor.transform(config.responseType, res);
        response.data = responseDataTransformed.data;
        if (responseDataTransformed.contentType && response.headers) {
            response.headers['Content-Type'] = responseDataTransformed.contentType;
        }
    }
    catch (err) {
        onError(helper_1.Utils.kyofuucError(err.message, config, helper_1.ErrorCode.RESPONSE_DATA_TRANSFORMATION_FAILED));
        return;
    }
    onSuccess(response);
}
let _defaultResponseTransformerRegistered = false;
if (!_defaultResponseTransformerRegistered) {
    _defaultResponseTransformerRegistered = true;
    exports.ResponseProcessor.register(exports.ResponseType.TEXT, textResponseTransformer);
    exports.ResponseProcessor.register(exports.ResponseType.JSON, jsonResponseTransformer);
    exports.ResponseProcessor.register(exports.ResponseType.BUFFER, bufferResponseTransformer);
    exports.ResponseProcessor.register(exports.ResponseType.STREAM, streamResponseTransformer);
    exports.ResponseProcessor.register(exports.ResponseType.NOCONTENT, noContentResponseTransformer);
    exports.ResponseProcessor.register(exports.ResponseType.ARRAY_BUFFER, arrayBufferResponseTransformer);
}
